@@ -273,29 +273,14 @@ _check_polkit_for_action (GsdDatetimeMechanism *mechanism, DBusGMethodInvocation
         return TRUE;
 }
 
-
 static gboolean
-_set_time (GsdDatetimeMechanism  *mechanism,
-           const struct timeval  *tv,
-           DBusGMethodInvocation *context)
+_sync_hwclock (DBusGMethodInvocation *context)
 {
         GError *error;
 
-        if (!_check_polkit_for_action (mechanism, context, "org.gnome.settingsdaemon.datetimemechanism.settime"))
-                return FALSE;
+        error = NULL;
 
-        if (settimeofday (tv, NULL) != 0) {
-                error = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
-                                     GSD_DATETIME_MECHANISM_ERROR_GENERAL,
-                                     "Error calling settimeofday({%lld,%lld}): %s", 
-                                     (gint64) tv->tv_sec, (gint64) tv->tv_usec,
-                                     strerror (errno));
-                dbus_g_method_return_error (context, error);
-                g_error_free (error);
-                return FALSE;
-        }
-
-        if (g_file_test ("/sbin/hwclock", 
+        if (g_file_test ("/sbin/hwclock",
                          G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_EXECUTABLE)) {
                 int exit_status;
                 if (!g_spawn_command_line_sync ("/sbin/hwclock --systohc", NULL, NULL, &exit_status, &error)) {
@@ -318,10 +303,88 @@ _set_time (GsdDatetimeMechanism  *mechanism,
                 }
         }
 
+        return TRUE;
+}
+
+static gboolean
+_set_time (GsdDatetimeMechanism  *mechanism,
+           const struct timeval  *tv,
+           DBusGMethodInvocation *context)
+{
+        GError *error;
+
+        if (!_check_polkit_for_action (mechanism, context, "org.gnome.settingsdaemon.datetimemechanism.settime"))
+                return FALSE;
+
+        if (settimeofday (tv, NULL) != 0) {
+                error = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
+                                     GSD_DATETIME_MECHANISM_ERROR_GENERAL,
+                                     "Error calling settimeofday({%lld,%lld}): %s",
+                                     (gint64) tv->tv_sec, (gint64) tv->tv_usec,
+                                     strerror (errno));
+                dbus_g_method_return_error (context, error);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        if (!_sync_hwclock (context))
+                return FALSE;
+
         dbus_g_method_return (context);
         return TRUE;
 }
 
+static gboolean
+_set_date (GsdDatetimeMechanism  *mechanism,
+           guint                  day,
+           guint                  month,
+           guint                  year,
+           DBusGMethodInvocation *context)
+{
+        GDateTime *time;
+        char *date_str, *time_str;
+        char *date_cmd;
+        int exit_status;
+        GError *error;
+
+        date_str = g_strdup_printf ("%02d/%02d/%d", month, day, year);
+        error = NULL;
+
+        time = g_date_time_new_now_local ();
+        time_str = g_date_time_format (time, "%R:%S");
+        g_date_time_unref (time);
+
+        date_cmd = g_strdup_printf ("/bin/date -s \"%s %s\" +\"%%D %%R:%%S\"", date_str, time_str);
+        g_free (date_str);
+        g_free (time_str);
+
+        if (!g_spawn_command_line_sync (date_cmd, NULL, NULL, &exit_status, &error)) {
+                GError *error2;
+                error2 = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
+                                      GSD_DATETIME_MECHANISM_ERROR_GENERAL,
+                                      "Error spawning /bin/date: %s", error->message);
+                g_error_free (error);
+                dbus_g_method_return_error (context, error2);
+                g_error_free (error2);
+                g_free (date_cmd);
+                return FALSE;
+        }
+        g_free (date_cmd);
+        if (WEXITSTATUS (exit_status) != 0) {
+                error = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
+                                     GSD_DATETIME_MECHANISM_ERROR_GENERAL,
+                                     "/bin/date returned %d", exit_status);
+                dbus_g_method_return_error (context, error);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        if (!_sync_hwclock (context))
+                return FALSE;
+
+        return TRUE;
+}
+
 static gboolean
 _rh_update_etc_sysconfig_clock (DBusGMethodInvocation *context, const char *key, const char *value)
 {
@@ -333,7 +396,7 @@ _rh_update_etc_sysconfig_clock (DBusGMethodInvocation *context, const char *key,
                 char *data;
                 gsize len;
                 GError *error;
-                
+
                 error = NULL;
 
                 if (!g_file_get_contents ("/etc/sysconfig/clock", &data, &len, &error)) {
@@ -397,13 +460,26 @@ gsd_datetime_mechanism_set_time (GsdDatetimeMechanism  *mechanism,
         struct timeval tv;
 
         reset_killtimer ();
-        g_debug ("SetTime(%lld) called", seconds_since_epoch);
+        g_debug ("SetTime(%" G_GINT64_FORMAT ") called", seconds_since_epoch);
 
         tv.tv_sec = (time_t) seconds_since_epoch;
         tv.tv_usec = 0;
         return _set_time (mechanism, &tv, context);
 }
 
+gboolean
+gsd_datetime_mechanism_set_date (GsdDatetimeMechanism  *mechanism,
+                                 guint                  day,
+                                 guint                  month,
+                                 guint                  year,
+                                 DBusGMethodInvocation *context)
+{
+        reset_killtimer ();
+        g_debug ("SetDate(%d, %d, %d) called", day, month, year);
+
+        return _set_date (mechanism, day, month, year, context);
+}
+
 gboolean
 gsd_datetime_mechanism_adjust_time (GsdDatetimeMechanism  *mechanism,
                                     gint64                 seconds_to_add,
@@ -412,7 +488,7 @@ gsd_datetime_mechanism_adjust_time (GsdDatetimeMechanism  *mechanism,
         struct timeval tv;
 
         reset_killtimer ();
-        g_debug ("AdjustTime(%lld) called", seconds_to_add);
+        g_debug ("AdjustTime(%" G_GINT64_FORMAT " ) called", seconds_to_add);
 
         if (gettimeofday (&tv, NULL) != 0) {
                 GError *error;
@@ -425,32 +501,65 @@ gsd_datetime_mechanism_adjust_time (GsdDatetimeMechanism  *mechanism,
         }
 
         tv.tv_sec += (time_t) seconds_to_add;
-        return _set_time (mechanism, &tv, context);        
+        return _set_time (mechanism, &tv, context);
 }
 
+static gboolean
+gsd_datetime_check_tz_name (const char *tz,
+                            GError    **error)
+{
+        GFile *file;
+        char *tz_path, *actual_path;
+        gboolean retval;
+
+        retval = TRUE;
+        tz_path = g_build_filename (SYSTEM_ZONEINFODIR, tz, NULL);
+
+        /* Get the actual resolved path */
+        file = g_file_new_for_path (tz);
+        actual_path = g_file_get_path (file);
+        g_object_unref (file);
+
+        /* The tz name passed had relative paths in it */
+        if (g_strcmp0 (tz_path, actual_path) != 0) {
+                g_set_error (error, GSD_DATETIME_MECHANISM_ERROR,
+                             GSD_DATETIME_MECHANISM_ERROR_INVALID_TIMEZONE_FILE,
+                             "Timezone file '%s' was invalid.",
+                             tz);
+                retval = FALSE;
+        }
+
+        g_free (tz_path);
+        g_free (actual_path);
+
+        return retval;
+}
 
 gboolean
 gsd_datetime_mechanism_set_timezone (GsdDatetimeMechanism  *mechanism,
-                                     const char            *zone_file,
+                                     const char            *tz,
                                      DBusGMethodInvocation *context)
 {
         GError *error;
 
         reset_killtimer ();
-        g_debug ("SetTimezone('%s') called", zone_file);
+        g_debug ("SetTimezone('%s') called", tz);
 
         if (!_check_polkit_for_action (mechanism, context, "org.gnome.settingsdaemon.datetimemechanism.settimezone"))
                 return FALSE;
 
         error = NULL;
 
-        if (!system_timezone_set_from_file (zone_file, &error)) {
+        if (!gsd_datetime_check_tz_name (tz, &error))
+                return FALSE;
+
+        if (!system_timezone_set (tz, &error)) {
                 GError *error2;
                 int     code;
 
                 if (error->code == SYSTEM_TIMEZONE_ERROR_INVALID_TIMEZONE_FILE)
                         code = GSD_DATETIME_MECHANISM_ERROR_INVALID_TIMEZONE_FILE;
-                else 
+                else
                         code = GSD_DATETIME_MECHANISM_ERROR_GENERAL;
 
                 error2 = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
@@ -585,6 +694,92 @@ gsd_datetime_mechanism_set_hardware_clock_using_utc (GsdDatetimeMechanism  *mech
         return TRUE;
 }
 
+gboolean
+gsd_datetime_mechanism_get_using_ntp  (GsdDatetimeMechanism    *mechanism,
+                                       DBusGMethodInvocation   *context)
+{
+        int exit_status;
+        GError *error = NULL;
+        gboolean can_use_ntp;
+        gboolean is_using_ntp;
+
+        if (g_file_test ("/etc/ntp.conf", G_FILE_TEST_EXISTS)) {
+                can_use_ntp = TRUE;
+                if (!g_spawn_command_line_sync ("/sbin/service ntpd status",
+                                                NULL, NULL, &exit_status, &error)) {
+                        GError *error2;
+                        error2 = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
+                                              GSD_DATETIME_MECHANISM_ERROR_GENERAL,
+                                              "Error spawning /sbin/service: %s", error->message);
+                        g_error_free (error);
+                        dbus_g_method_return_error (context, error2);
+                        g_error_free (error2);
+                        return FALSE;
+                }
+                if (exit_status == 0)
+                        is_using_ntp = TRUE;
+                else
+                        is_using_ntp = FALSE;
+        }
+        else {
+                can_use_ntp = FALSE;
+                is_using_ntp = FALSE;
+        }
+
+        dbus_g_method_return (context, can_use_ntp, is_using_ntp);
+        return TRUE;
+}
+
+gboolean
+gsd_datetime_mechanism_set_using_ntp  (GsdDatetimeMechanism    *mechanism,
+                                       gboolean                 using_ntp,
+                                       DBusGMethodInvocation   *context)
+{
+        GError *error;
+        int exit_status;
+        char *cmd;
+
+        error = NULL;
+
+        if (!_check_polkit_for_action (mechanism, context, "org.gnome.clockapplet.mechanism.configurentp"))
+                return FALSE;
+
+        cmd = g_strconcat ("/sbin/chkconfig --level 2345 ntpd ", using_ntp ? "on" : "off", NULL);
+        if (!g_spawn_command_line_sync (cmd,
+                                        NULL, NULL, &exit_status, &error)) {
+                GError *error2;
+                error2 = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
+                                      GSD_DATETIME_MECHANISM_ERROR_GENERAL,
+                                      "Error spawning /sbin/chkconfig: %s", error->message);
+                g_error_free (error);
+                dbus_g_method_return_error (context, error2);
+                g_error_free (error2);
+                g_free (cmd);
+                return FALSE;
+        }
+
+        g_free (cmd);
+
+        cmd = g_strconcat ("/sbin/service ntpd ", using_ntp ? "restart" : "stop", NULL);
+        if (!g_spawn_command_line_sync (cmd,
+                                        NULL, NULL, &exit_status, &error)) {
+                GError *error2;
+                error2 = g_error_new (GSD_DATETIME_MECHANISM_ERROR,
+                                      GSD_DATETIME_MECHANISM_ERROR_GENERAL,
+                                      "Error spawning /sbin/service: %s", error->message);
+                g_error_free (error);
+                dbus_g_method_return_error (context, error2);
+                g_error_free (error2);
+                g_free (cmd);
+                return FALSE;
+        }
+
+        g_free (cmd);
+
+        dbus_g_method_return (context);
+        return TRUE;
+}
+
 static void
 check_can_do (GsdDatetimeMechanism  *mechanism,
               const char            *action,
@@ -650,3 +845,14 @@ gsd_datetime_mechanism_can_set_timezone (GsdDatetimeMechanism  *mechanism,
 
         return TRUE;
 }
+
+gboolean
+gsd_datetime_mechanism_can_set_using_ntp (GsdDatetimeMechanism  *mechanism,
+                                          DBusGMethodInvocation *context)
+{
+        check_can_do (mechanism,
+                      "org.gnome.settingsdaemon.datetimemechanism.configurentp",
+                      context);
+
+        return TRUE;
+}
