@@ -47,32 +47,34 @@
 
 #include "gnome-settings-profile.h"
 #include "gsd-keyboard-manager.h"
+#include "gsd-enums.h"
 
 #include "gsd-keyboard-xkb.h"
-#include "gsd-xmodmap.h"
 
 #define GSD_KEYBOARD_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_KEYBOARD_MANAGER, GsdKeyboardManagerPrivate))
 
-#define GSD_KEYBOARD_KEY "/desktop/gnome/peripherals/keyboard"
+#ifndef HOST_NAME_MAX
+#  define HOST_NAME_MAX 255
+#endif
 
-#define KEY_REPEAT        "/desktop/gnome/peripherals/keyboard/repeat"
-#define KEY_CLICK         "/desktop/gnome/peripherals/keyboard/click"
-#define KEY_RATE          "/desktop/gnome/peripherals/keyboard/rate"
-#define KEY_DELAY         "/desktop/gnome/peripherals/keyboard/delay"
-#define KEY_CLICK_VOLUME  "/desktop/gnome/peripherals/keyboard/click_volume"
+#define GSD_KEYBOARD_DIR "org.gnome.settings-daemon.peripherals.keyboard"
 
-#define KEY_BELL_VOLUME   "/desktop/gnome/peripherals/keyboard/bell_volume"
-#define KEY_BELL_PITCH    "/desktop/gnome/peripherals/keyboard/bell_pitch"
-#define KEY_BELL_DURATION "/desktop/gnome/peripherals/keyboard/bell_duration"
-#define KEY_BELL_MODE     "/desktop/gnome/peripherals/keyboard/bell_mode"
+#define KEY_REPEAT         "repeat"
+#define KEY_CLICK          "click"
+#define KEY_RATE           "rate"
+#define KEY_DELAY          "delay"
+#define KEY_CLICK_VOLUME   "click-volume"
+
+#define KEY_BELL_VOLUME    "bell-volume"
+#define KEY_BELL_PITCH     "bell-pitch"
+#define KEY_BELL_DURATION  "bell-duration"
+#define KEY_BELL_MODE      "bell-mode"
 
 struct GsdKeyboardManagerPrivate
 {
-        gboolean dummy;
-};
-
-enum {
-        PROP_0,
+        GSettings *settings;
+        gboolean   have_xkb;
+        gint       xkb_event_base;
 };
 
 static void     gsd_keyboard_manager_class_init  (GsdKeyboardManagerClass *klass);
@@ -83,7 +85,6 @@ G_DEFINE_TYPE (GsdKeyboardManager, gsd_keyboard_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
-
 #ifdef HAVE_X11_EXTENSIONS_XF86MISC_H
 static gboolean
 xfree86_set_keyboard_autorepeat_rate (int delay, int rate)
@@ -92,23 +93,24 @@ xfree86_set_keyboard_autorepeat_rate (int delay, int rate)
         int      event_base_return;
         int      error_base_return;
 
-        if (XF86MiscQueryExtension (GDK_DISPLAY (),
+        if (XF86MiscQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                     &event_base_return,
                                     &error_base_return) == True) {
                 /* load the current settings */
                 XF86MiscKbdSettings kbdsettings;
-                XF86MiscGetKbdSettings (GDK_DISPLAY (), &kbdsettings);
+                XF86MiscGetKbdSettings (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &kbdsettings);
 
                 /* assign the new values */
                 kbdsettings.delay = delay;
                 kbdsettings.rate = rate;
-                XF86MiscSetKbdSettings (GDK_DISPLAY (), &kbdsettings);
+                XF86MiscSetKbdSettings (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &kbdsettings);
                 res = TRUE;
         }
 
         return res;
 }
 #endif /* HAVE_X11_EXTENSIONS_XF86MISC_H */
+
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
 static gboolean
 xkb_set_keyboard_autorepeat_rate (int delay, int rate)
@@ -116,7 +118,7 @@ xkb_set_keyboard_autorepeat_rate (int delay, int rate)
         int interval = (rate <= 0) ? 1000000 : 1000/rate;
         if (delay <= 0)
                 delay = 1;
-        return XkbSetAutoRepeatRate (GDK_DISPLAY (),
+        return XkbSetAutoRepeatRate (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                      XkbUseCoreKbd,
                                      delay,
                                      interval);
@@ -124,55 +126,68 @@ xkb_set_keyboard_autorepeat_rate (int delay, int rate)
 #endif
 
 static char *
-gsd_keyboard_get_hostname_key (const char *subkey)
+gsd_keyboard_get_hostname_key (void)
 {
-#ifdef HOST_NAME_MAX
-        char hostname[HOST_NAME_MAX + 1];
-#else
-        char hostname[256];
-#endif
+        /* FIXME disabled for now, as we need GSettingsList support:
+         * https://bugzilla.gnome.org/show_bug.cgi?id=622126 */
+#if 0
+        const char *hostname;
 
-        if (gethostname (hostname, sizeof (hostname)) == 0 &&
-            strcmp (hostname, "localhost") != 0 &&
-            strcmp (hostname, "localhost.localdomain") != 0) {
+        hostname = g_get_host_name ();
+
+        if (g_str_equal (hostname, "localhost") == FALSE &&
+            g_str_equal (hostname, "localhost.localdomain") == FALSE) {
                 char *escaped;
                 char *key;
 
-                escaped = gconf_escape_key (hostname, -1);
-                key = g_strconcat (GSD_KEYBOARD_KEY
-                                   "/host-",
-                                   escaped,
-                                   "/0/",
-                                   subkey,
-                                   (char *)NULL);
+                /* FIXME, really escape? */
+                escaped = g_strdup (hostname);
+                key = g_strdup_printf ("host-%s-0-numlock-on", escaped);
                 g_free (escaped);
                 return key;
-        } else
+        } else {
+                g_message ("NumLock remembering disabled because hostname is set to \"localhost\"");
                 return NULL;
+        }
+#endif
+        return NULL;
 }
 
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
 
-enum {
+typedef enum {
         NUMLOCK_STATE_OFF = 0,
         NUMLOCK_STATE_ON = 1,
         NUMLOCK_STATE_UNKNOWN = 2
-};
+} NumLockState;
 
-/* something fatal has happened so that it makes no
- * sense to try to remember anything.
- * that means: no calls to the set_state functions!
- */
-static gboolean
-numlock_setup_error = FALSE;
+static void
+numlock_xkb_init (GsdKeyboardManager *manager)
+{
+        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+        gboolean have_xkb;
+        int opcode, error_base, major, minor;
 
-/* we didn't apply GConf settings yet
- * don't overwrite them with the initial state from
- * the newly started session!
- */
-static gboolean
-numlock_starting_up = TRUE;
+        have_xkb = XkbQueryExtension (dpy,
+                                      &opcode,
+                                      &manager->priv->xkb_event_base,
+                                      &error_base,
+                                      &major,
+                                      &minor)
+                && XkbUseExtension (dpy, &major, &minor);
 
+        if (have_xkb) {
+                XkbSelectEventDetails (dpy,
+                                       XkbUseCoreKbd,
+                                       XkbStateNotifyMask,
+                                       XkbModifierLockMask,
+                                       XkbModifierLockMask);
+        } else {
+                g_warning ("XKB extension not available");
+        }
+
+        manager->priv->have_xkb = have_xkb;
+}
 
 static unsigned
 numlock_NumLock_modifier_mask (void)
@@ -182,7 +197,7 @@ numlock_NumLock_modifier_mask (void)
 }
 
 static void
-numlock_set_xkb_state (gboolean new_state)
+numlock_set_xkb_state (NumLockState new_state)
 {
         unsigned int num_mask;
         Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
@@ -192,120 +207,77 @@ numlock_set_xkb_state (gboolean new_state)
         XkbLockModifiers (dpy, XkbUseCoreKbd, num_mask, new_state ? num_mask : 0);
 }
 
-static char *
-numlock_gconf_state_key (void)
+static NumLockState
+numlock_get_gsettings_state (GSettings *settings)
 {
-        char *key = gsd_keyboard_get_hostname_key ("numlock_on");
-        if (!key) {
-                numlock_setup_error = TRUE;
-                g_warning ("numlock: Numlock remembering disabled because your hostname is set to \"localhost\".");
-        }
-        return key;
-}
+        int   curr_state;
+        char *key;
 
-static int
-numlock_get_gconf_state (GConfClient *client)
-{
-        int          curr_state;
-        GError      *err = NULL;
-        char        *key = numlock_gconf_state_key ();
-
-        if (!key) {
+        key = gsd_keyboard_get_hostname_key ();
+        if (!key)
                 return NUMLOCK_STATE_UNKNOWN;
-        }
 
-        curr_state = gconf_client_get_bool (client, key, &err);
-        if (err) {
-                curr_state = NUMLOCK_STATE_UNKNOWN;
-        }
+        curr_state = g_settings_get_boolean (settings, key);
 
-        g_clear_error (&err);
         g_free (key);
+
         return curr_state;
 }
 
 static void
-numlock_set_gconf_state (GConfClient *client,
-                         gboolean     new_state)
+numlock_set_gsettings_state (GSettings   *settings,
+                             NumLockState new_state)
 {
         char *key;
 
         if (new_state != NUMLOCK_STATE_ON && new_state != NUMLOCK_STATE_OFF) {
                 return;
         }
-        key = numlock_gconf_state_key ();
-        if (!key) {
-                return;
-        }
 
-        gconf_client_set_bool (client, key, new_state, NULL);
-        g_free (key);
+        key = gsd_keyboard_get_hostname_key ();
+        if (key) {
+                g_settings_set_boolean (settings, key, new_state);
+                g_free (key);
+        }
 }
 
 static GdkFilterReturn
 numlock_xkb_callback (GdkXEvent *xev_,
-                      GdkEvent *gdkev_,
-                      gpointer xkb_event_code)
+                      GdkEvent  *gdkev_,
+                      gpointer   user_data)
 {
-        XEvent *xev = (XEvent *)xev_;
+        XEvent *xev = (XEvent *) xev_;
+        GsdKeyboardManager *manager = (GsdKeyboardManager *) user_data;
 
-        if (xev->type == GPOINTER_TO_INT (xkb_event_code)) {
+        if (xev->type == manager->priv->xkb_event_base) {
                 XkbEvent *xkbev = (XkbEvent *)xev;
                 if (xkbev->any.xkb_type == XkbStateNotify)
                 if (xkbev->state.changed & XkbModifierLockMask) {
                         unsigned num_mask = numlock_NumLock_modifier_mask ();
                         unsigned locked_mods = xkbev->state.locked_mods;
                         int numlock_state = !! (num_mask & locked_mods);
-
-                        if (!numlock_starting_up && !numlock_setup_error) {
-                                GConfClient *client;
-                                client = gconf_client_get_default ();
-                                numlock_set_gconf_state (client, numlock_state);
-                                g_object_unref (client);
-                        }
+                        numlock_set_gsettings_state (manager->priv->settings, numlock_state);
                 }
         }
         return GDK_FILTER_CONTINUE;
 }
 
 static void
-numlock_install_xkb_callback (void)
+numlock_install_xkb_callback (GsdKeyboardManager *manager)
 {
-        Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        int      op_code = 0;
-        int      xkb_event_code = 0;
-        int      error_code = 0;
-        int      major = XkbMajorVersion;
-        int      minor = XkbMinorVersion;
-        int      have_xkb = XkbQueryExtension (dpy,
-                                          &op_code,
-                                          &xkb_event_code,
-                                          &error_code,
-                                          &major,
-                                          &minor);
-        if (have_xkb != True) {
-                numlock_setup_error = TRUE;
-                g_warning ("numlock: XkbQueryExtension returned an error");
+        if (!manager->priv->have_xkb)
                 return;
-        }
-
-        XkbSelectEventDetails (dpy,
-                               XkbUseCoreKbd,
-                               XkbStateNotifyMask,
-                               XkbModifierLockMask,
-                               XkbModifierLockMask);
 
         gdk_window_add_filter (NULL,
                                numlock_xkb_callback,
-                               GINT_TO_POINTER (xkb_event_code));
+                               manager);
 }
 
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
 static void
-apply_settings (GConfClient        *client,
-                guint               cnxn_id,
-                GConfEntry         *entry,
+apply_settings (GSettings          *settings,
+                const char         *key,
                 GsdKeyboardManager *manager)
 {
         XKeyboardControl kbdcontrol;
@@ -317,37 +289,33 @@ apply_settings (GConfClient        *client,
         int              bell_volume;
         int              bell_pitch;
         int              bell_duration;
-        char            *volume_string;
+        GsdBellMode      bell_mode;
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
         gboolean         rnumlock;
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
-        repeat        = gconf_client_get_bool  (client, KEY_REPEAT, NULL);
-        click         = gconf_client_get_bool  (client, KEY_CLICK, NULL);
-        rate          = gconf_client_get_int   (client, KEY_RATE, NULL);
-        delay         = gconf_client_get_int   (client, KEY_DELAY, NULL);
-        click_volume  = gconf_client_get_int   (client, KEY_CLICK_VOLUME, NULL);
-#if 0
-        bell_volume   = gconf_client_get_int   (client, KEY_BELL_VOLUME, NULL);
-#endif
-        bell_pitch    = gconf_client_get_int   (client, KEY_BELL_PITCH, NULL);
-        bell_duration = gconf_client_get_int   (client, KEY_BELL_DURATION, NULL);
+        repeat        = g_settings_get_boolean  (settings, KEY_REPEAT);
+        click         = g_settings_get_boolean  (settings, KEY_CLICK);
+        rate          = g_settings_get_int   (settings, KEY_RATE);
+        delay         = g_settings_get_int   (settings, KEY_DELAY);
+        click_volume  = g_settings_get_int   (settings, KEY_CLICK_VOLUME);
+        bell_pitch    = g_settings_get_int   (settings, KEY_BELL_PITCH);
+        bell_duration = g_settings_get_int   (settings, KEY_BELL_DURATION);
 
-        volume_string = gconf_client_get_string (client, KEY_BELL_MODE, NULL);
-        bell_volume   = (volume_string && !strcmp (volume_string, "on")) ? 50 : 0;
-        g_free (volume_string);
+        bell_mode = g_settings_get_enum (settings, KEY_BELL_MODE);
+        bell_volume   = (bell_mode == GSD_BELL_MODE_ON) ? 50 : 0;
 
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
-        rnumlock      = gconf_client_get_bool  (client, GSD_KEYBOARD_KEY "/remember_numlock_state", NULL);
+        rnumlock      = g_settings_get_boolean  (settings, "remember-numlock-state");
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         gdk_error_trap_push ();
         if (repeat) {
                 gboolean rate_set = FALSE;
 
-                XAutoRepeatOn (GDK_DISPLAY ());
+                XAutoRepeatOn (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
                 /* Use XKB in preference */
-#if defined (HAVE_X11_EXTENSIONS_XKB_H)
+#ifdef HAVE_X11_EXTENSIONS_XKB_H
                 rate_set = xkb_set_keyboard_autorepeat_rate (delay, rate);
 #endif
 #ifdef HAVE_X11_EXTENSIONS_XF86MISC_H
@@ -358,7 +326,7 @@ apply_settings (GConfClient        *client,
                         g_warning ("Neither XKeyboard not Xfree86's keyboard extensions are available,\n"
                                    "no way to support keyboard autorepeat rate settings");
         } else {
-                XAutoRepeatOff (GDK_DISPLAY ());
+                XAutoRepeatOff (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
         }
 
         /* as percentage from 0..100 inclusive */
@@ -371,64 +339,65 @@ apply_settings (GConfClient        *client,
         kbdcontrol.bell_percent = bell_volume;
         kbdcontrol.bell_pitch = bell_pitch;
         kbdcontrol.bell_duration = bell_duration;
-        XChangeKeyboardControl (GDK_DISPLAY (),
+        XChangeKeyboardControl (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                                 KBKeyClickPercent | KBBellPercent | KBBellPitch | KBBellDuration,
                                 &kbdcontrol);
 
-
 #ifdef HAVE_X11_EXTENSIONS_XKB_H
-        if (!numlock_setup_error && rnumlock) {
-                GConfClient *client;
-                client = gconf_client_get_default ();
-                numlock_set_xkb_state (numlock_get_gconf_state (client));
-                g_object_unref (client);
+        if (manager->priv->have_xkb && rnumlock) {
+                numlock_set_xkb_state (numlock_get_gsettings_state (manager->priv->settings));
         }
-        numlock_starting_up = FALSE;
 #endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
-        XSync (GDK_DISPLAY (), FALSE);
-        gdk_error_trap_pop ();
+        XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
+        gdk_error_trap_pop_ignored ();
 }
 
-static void
-register_config_callback (GsdKeyboardManager      *manager,
-                          const char              *path,
-                          GConfClientNotifyFunc    func)
+void
+gsd_keyboard_manager_apply_settings (GsdKeyboardManager *manager)
 {
-        GConfClient *client;
+        apply_settings (manager->priv->settings, NULL, manager);
+}
 
-        client = gconf_client_get_default ();
+static gboolean
+start_keyboard_idle_cb (GsdKeyboardManager *manager)
+{
+        gnome_settings_profile_start (NULL);
 
-        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
-        gconf_client_notify_add (client, path, func, manager, NULL, NULL);
+        g_debug ("Starting keyboard manager");
 
-        g_object_unref (client);
+        manager->priv->have_xkb = 0;
+        manager->priv->settings = g_settings_new (GSD_KEYBOARD_DIR);
+
+        /* Essential - xkb initialization should happen before */
+        gsd_keyboard_xkb_init (manager);
+
+#ifdef HAVE_X11_EXTENSIONS_XKB_H
+        numlock_xkb_init (manager);
+#endif /* HAVE_X11_EXTENSIONS_XKB_H */
+
+        /* apply current settings before we install the callback */
+        gsd_keyboard_manager_apply_settings (manager);
+
+        g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
+                          G_CALLBACK (apply_settings), manager);
+
+#ifdef HAVE_X11_EXTENSIONS_XKB_H
+        numlock_install_xkb_callback (manager);
+#endif /* HAVE_X11_EXTENSIONS_XKB_H */
+
+        gnome_settings_profile_end (NULL);
+
+        return FALSE;
 }
 
 gboolean
 gsd_keyboard_manager_start (GsdKeyboardManager *manager,
                             GError            **error)
 {
-        GConfClient *client;
-
         gnome_settings_profile_start (NULL);
 
-        client = gconf_client_get_default ();
-        g_debug ("Starting keyboard manager");
-        /* Essential - xkb initialization should happen before */
-        gsd_keyboard_xkb_set_post_activation_callback ((PostActivationCallback) gsd_load_modmap_files, NULL);
-        gsd_keyboard_xkb_init (client);
-        gsd_keyboard_xkb_load (client);
-
-        register_config_callback (manager,
-                                  GSD_KEYBOARD_KEY,
-                                  (GConfClientNotifyFunc)apply_settings);
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
-        numlock_install_xkb_callback ();
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
-
-        apply_settings (client, 0, NULL, manager);
-        g_object_unref (client);
+        g_idle_add ((GSourceFunc) start_keyboard_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -438,7 +407,24 @@ gsd_keyboard_manager_start (GsdKeyboardManager *manager,
 void
 gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
 {
+        GsdKeyboardManagerPrivate *p = manager->priv;
+
         g_debug ("Stopping keyboard manager");
+
+        if (p->settings != NULL) {
+                g_object_unref (p->settings);
+                p->settings = NULL;
+        }
+
+#if HAVE_X11_EXTENSIONS_XKB_H
+        if (p->have_xkb) {
+                gdk_window_remove_filter (NULL,
+                                          numlock_xkb_callback,
+                                          manager);
+        }
+#endif /* HAVE_X11_EXTENSIONS_XKB_H */
+
+        gsd_keyboard_xkb_shutdown ();
 }
 
 static void
@@ -520,7 +506,6 @@ static void
 gsd_keyboard_manager_init (GsdKeyboardManager *manager)
 {
         manager->priv = GSD_KEYBOARD_MANAGER_GET_PRIVATE (manager);
-
 }
 
 static void
